@@ -1,99 +1,90 @@
 import os
-import sys
-from collections import defaultdict
-
 import numpy as np
 import argparse
 import json
 from collections import defaultdict
 from tqdm import tqdm
-
 from intervaltree import IntervalTree, Interval
-
-from docopt import docopt
-
-from profane import config_list_to_dict, constants, ConfigOption, Dependency, ModuleBase
-
 from mako.template import Template
 from rich.console import Console
 from rich.table import Table
-from rich import print as rprint
 from rich.prompt import Confirm
-from rich.live import Live
 from rich.panel import Panel
-from rich.layout import Layout
 import ir_datasets
+from ir_measures import iter_calc
+from ir_measures import P, nDCG
+from diffir import QrelMeasure, TopkMeasure, WeightBuilder
 from diffir.utils import load_trec_run
 
 _logger = ir_datasets.log.easy()
 
-# specify a base package that profane should look for modules under
-constants["BASE_PACKAGE"] = "diffir"
-
 
 def main():
-    help = """
-            Usage:
-              run.py RUN1 RUN2 [options] [([with] CONFIG...)]
-              run.py (-h | --help)
-
-
-            Options:
-              -c --cli                      CLI mode (default)
-              -w --web                      webui mode
-              -h --help                     Print this help message and exit.
-
-
-            Arguments:
-              RUN1      First run file to compare
-              RUN2      Second run file to compare
-              CONFIG    Configuration assignments of the form foo.bar=17
-
-           """
-
     parser = argparse.ArgumentParser()
-    parser.add_argument("runfiles", nargs="+")
-    parser.add_argument("-c", "--cli", dest="cli", action="store_true")
-    parser.add_argument("-w", "--web", dest="web", action="store_true")
-    parser.add_argument("--config", dest="config", nargs="*")
-
+    parser.add_argument("runfiles", nargs="+", help="run file(s) to display and compare")
+    parser.add_argument("-c", "--cli", dest="cli", action="store_true", help="output to CLI (default)")
+    parser.add_argument("-w", "--web", dest="web", action="store_true", help="output HTML file for WebUI")
+    parser.add_argument("--dataset", dest="dataset", type=str, required=True, help="dataset identifier from ir_datasets")
+    parser.add_argument(
+        "--measure", dest="measure", type=str, default="tauap", help="measure for ranking difference (qrel, tauap, weightedtau)"
+    )
+    parser.add_argument(
+        "--metric", dest="metric", type=str, default="nDCG@10", help="metric to report and used with qrel measure"
+    )
+    parser.add_argument("--topk", dest="topk", type=int, default=50, help="number of queries to compare")
+    parser.add_argument("--weights_1", dest="weights_1", type=str, default=None, required=False)
+    parser.add_argument("--weights_2", dest="weights_2", type=str, default=None, required=False)
     args = parser.parse_args()
-    diff(args.runfiles, args.config, cli=args.cli, web=args.web)
+    config = {
+        "dataset": args.dataset,
+        "measure": args.measure,
+        "metric": args.metric,
+        "topk": args.topk,
+        "weight": {"weights_1": args.weights_1, "weights_2": args.weights_2},
+    }
+    if not (args.cli or args.web):
+        args.cli = True  # default
+    diff(args.runfiles, config, cli=args.cli, web=args.web)
 
 
 def diff(runs, config, cli, web, print_html=True):
-    config = config_list_to_dict(config) if config else {}
-
-    # hack to automatically add weight files when available and not already specified
-    config_with_defaults = MainTask(config, build=False).config
     for i, run in enumerate(runs):
-        if f"weights_{i+1}" in config_with_defaults["weight"] and config_with_defaults["weight"][f"weights_{i+1}"] is None:
+        if config["weight"][f"weights_{i + 1}"] is None:
             if os.path.exists(run + ".diffir"):
-                config.setdefault("weight", {})[f"weights_{i+1}"] = run + ".diffir"
-
-    task = MainTask(config)
+                _logger.info("Found weight file at {}".format(run + ".diffir"))
+                config["weight"][f"weights_{i + 1}"] = run + ".diffir"
+            else:
+                _logger.info("No weight file for {}. Fall to exact matching".format(run))
+    task = MainTask(**config)
     if cli:
         task.cli(runs)
     if web:
         html = task.web(runs)
         if print_html:
             print(html)
-        return task.config, html
+        return config, html
 
 
-class MainTask(ModuleBase):
+class MainTask:
     module_type = "task"
     module_name = "main"
-    config_spec = [
-        ConfigOption(key="dataset", default_value="none", description="TODO"),
-        ConfigOption(key="queries", default_value="none", description="TODO"),
-    ]
-    dependencies = [
-        Dependency(key="measure", module="measure", name="topk"),
-        Dependency(key="weight", module="weight", name="exactmatch"),
-    ]
 
-    def create_query_objects(self, run_1, run_2, qids, dataset):
+    def __init__(self, dataset="none", queries="none", measure="topk", metric="weighted_tau", topk=3, weight={}):
+        self.dataset = dataset
+        self.queries = queries
+        if measure == "qrel":
+            self.measure = QrelMeasure(metric, topk)
+        elif measure in ["tauap", "weightedtau", "spearmanr", "pearsonrank", "kldiv"]:
+            self.measure = TopkMeasure(measure, topk)
+        else:
+            raise ValueError("Measure {} is not supported".format(measure))
+
+        self.weight = WeightBuilder(weight["weights_1"], weight["weights_2"])
+
+    def compute_qrel_metrics(self):
+        pass
+
+    def create_query_objects(self, run_1, run_2, qids, qid2diff, metric_name, dataset, qid2qrelscores=None):
         """
         TODO: Need a better name
         This method takes in 2 runs and a set of qids, and constructs a dict for each qid (format specified below)
@@ -121,9 +112,14 @@ class MainTask(ModuleBase):
         """
         assert dataset.has_qrels(), "Cannot determine whether the doc is relevant - need qrels"
         qrels = dataset.qrels_dict()
-
+        run1_metrics = defaultdict(lambda: defaultdict(lambda: None))
+        for metrics in iter_calc([P@1, P@3, P@5, P@10, nDCG@1, nDCG@3, nDCG@5, nDCG@10], qrels, run_1):
+            run1_metrics[metrics.query_id][str(metrics.measure)] = metrics.value
+        if run_2:
+            run2_metrics = defaultdict(lambda: defaultdict(lambda: None))
+            for metrics in iter_calc([P@1, P@3, P@5, P@10, nDCG@1, nDCG@3, nDCG@5, nDCG@10], qrels, run_2):
+                run2_metrics[metrics.query_id][str(metrics.measure)] = metrics.value
         docstore = dataset.docs_store()
-
         qids_set = set(qids)  # Sets do O(1) lookups
         qid2object = {}
         for query in tqdm(dataset.queries_iter(), desc="analyzing queries"):
@@ -138,6 +134,10 @@ class MainTask(ModuleBase):
             )
 
             fields = query._asdict()
+            fields["contrast"] = {"name": metric_name, "value": qid2diff[query.query_id]}
+            if qid2qrelscores:
+                fields[f"Run1 {metric_name}"] = qid2qrelscores[query.query_id][0]
+                fields[f"Run2 {metric_name}"] = qid2qrelscores[query.query_id][1]
             qrels_for_query = qrels.get(query.query_id, {})
             run_1_for_query = []
             for rank, (doc_id, score) in enumerate(run_1[query.query_id].items()):
@@ -177,13 +177,84 @@ class MainTask(ModuleBase):
 
             qid2object[query.query_id] = {
                 "fields": fields,
+                "metrics": {
+                    metric: [run1_metrics[query.query_id][metric], run2_metrics[query.query_id][metric]]
+                    if run_2
+                    else [run1_metrics[query.query_id][metric]]
+                    for metric in ["P@1", "P@3", "P@5", "P@10", "nDCG@1", "nDCG@3", "nDCG@5", "nDCG@10"]
+                },
                 "run_1": run_1_for_query,
                 "run_2": run_2_for_query,
+                "summary": self.create_summary(run_1_for_query, run_2_for_query),
                 "mergedWeights": self.merge_weights(run_1_for_query, run_2_for_query),
             }
 
         return [qid2object[id] for id in qids]
 
+    def create_summary(self, run1_ranked_docs, run2_ranked_docs):
+        summary = []
+        if len(run2_ranked_docs) == 0:
+            unjudged_count = 0
+            for doc in run1_ranked_docs:
+                if doc["relevance"] is None:
+                    unjudged_count += 1
+            if unjudged_count > 0:
+                summary.append(["{} unjudged doc(s) move to a top spot in the ranking".format(unjudged_count)])
+            return summary
+
+        run1_doc_map = {doc["doc_id"]: doc for doc in run1_ranked_docs}
+        run2_doc_map = {doc["doc_id"]: doc for doc in run2_ranked_docs}
+        not_ranked_in_run2 = 0
+        for doc_id in run1_doc_map:
+            if doc_id not in run2_doc_map:
+                not_ranked_in_run2 += 1
+        not_ranked_in_run1 = 0
+        for doc_id in run2_doc_map:
+            if doc_id not in run1_doc_map:
+                not_ranked_in_run1 += 1
+        not_ranks = []
+        if not_ranked_in_run2 > 0:
+            not_ranks.append("{} document(s) in run1 not ranked in run2".format(not_ranked_in_run2))
+        if not_ranked_in_run1 > 0:
+            not_ranks.append("{} document(s) in run2 not ranked in run1".format(not_ranked_in_run1))
+        summary.append(not_ranks)
+        moves = []
+        for doc_id in run1_doc_map:
+            if doc_id not in run2_doc_map:
+                continue
+            rank_in_1 = run1_doc_map[doc_id]["rank"]
+            rank_in_2 = run2_doc_map[doc_id]["rank"]
+            if rank_in_1 != rank_in_2:
+                moves.append(("#{} in run1 moves to #{} in run2".format(rank_in_1, rank_in_2), abs(rank_in_1 - rank_in_2)))
+        moves = sorted(moves, key=lambda x: x[1], reverse=True)
+        if len(moves) > 0:
+            summary.append([move[0] for move in moves[:5]])
+        unjudged_run1 = 0
+        unjudged_run2 = 0
+        for doc_id in run1_doc_map:
+            if run1_doc_map[doc_id]["relevance"] is None and run1_doc_map[doc_id]["rank"] <= 10:
+                unjudged_run1 += 1
+        for doc_id in run2_doc_map:
+            if run2_doc_map[doc_id]["relevance"] is None and run2_doc_map[doc_id]["rank"] <= 10:
+                unjudged_run2 += 1
+        unjudged = []
+        if unjudged_run1 > 0:
+            unjudged.append("{} unjudged docs in run1 move to a top spot in the ranking".format(unjudged_run1))
+        if unjudged_run2 > 0:
+            unjudged.append("{} unjudged docs in run2 move to a top spot in the ranking".format(unjudged_run2))
+        summary.append(unjudged)
+        swap = 0
+        doc_ids = list(set(run1_doc_map.keys()).intersection(set(run2_doc_map.keys())))
+        for idx1 in range(len(doc_ids)):
+            for idx2 in range(idx1 + 1, len(doc_ids)):
+                if (run1_doc_map[doc_ids[idx1]]["rank"] - run1_doc_map[doc_ids[idx2]]["rank"]) * (
+                    run2_doc_map[doc_ids[idx1]]["rank"] - run2_doc_map[doc_ids[idx2]]["rank"]
+                ) < 0:
+                    swap += 1
+        if swap > 0:
+            summary.append(["Number of relative rank reversals: {}".format(swap)])
+        return summary
+
     def merge_weights(self, run1_for_query, run_2_for_query):
         doc_id2weights = defaultdict(lambda: {"run1": defaultdict(lambda: []), "run2": defaultdict(lambda: [])})
 
@@ -306,15 +377,14 @@ class MainTask(ModuleBase):
         run_1 = load_trec_run(run_1_fn)
         run_2 = load_trec_run(run_2_fn) if run_2_fn is not None else None
 
-        dataset = ir_datasets.load(self.config["dataset"])
-        assert dataset.has_docs()
-        assert dataset.has_queries()
-        # TODO: handle the case without qrels
-        assert dataset.has_queries()
-
-        diff_queries = self.measure.query_differences(run_1, run_2, dataset=dataset)
+        dataset = ir_datasets.load(self.dataset)
+        assert dataset.has_docs(), "dataset has no documents; maybe you're missing a partition like '/trec-dl-2020'"
+        assert dataset.has_queries(), "dataset has no queries; maybe you're missing a partition like '/trec-dl-2020'"
+        diff_queries, qid2diff, metric_name, qid2qrelscores = self.measure.query_differences(run_1, run_2, dataset=dataset)
         # _logger.info(diff_queries)
-        diff_query_objects = self.create_query_objects(run_1, run_2, diff_queries, dataset)
+        diff_query_objects = self.create_query_objects(
+            run_1, run_2, diff_queries, qid2diff, metric_name, dataset, qid2qrelscores=qid2qrelscores
+        )
         doc_objects = self.create_doc_objects(diff_query_objects, dataset)
 
         return json.dumps(
@@ -322,9 +392,9 @@ class MainTask(ModuleBase):
                 "meta": {
                     "run1_name": run_1_fn,
                     "run2_name": run_2_fn,
-                    "dataset": self.config["dataset"],
+                    "dataset": self.dataset,
                     "measure": self.measure.module_name,
-                    "weight": self.weight.module_name,
+                    # "weight": self.weight.module_name,
                     "qrelDefs": dataset.qrels_defs(),
                     "queryFields": dataset.queries_cls()._fields,
                     "docFields": dataset.docs_cls()._fields,
